﻿using System;
using Ucloudbiz.Storage;

namespace sample
{
    class Program
    {
        private const string TestContainerName = "testcontainer";
        private const string LocalFileName = "newobject.txt";
        private const string ObjectName = "newobject.txt";
        private const string NameKey = "name";

        static void Main(string[] args)
        {
            var creds = new KT_UserCredentials("YOUR_USERNAME_HERE", "YOUR_API_KEY_HERE");
            var client = new KT_Client();
            KT_Connection conn = new KT_Connection(creds, client);
            Console.WriteLine("Authenticating...");
            conn.Authenticate();
            Console.WriteLine("Authenticated");

            ListAllContainers(conn, client);
            CreateContainer(conn, client);
            SaveObjectToCloud(conn, client); 
            SaveObjectLocally(conn, client);
            ListAllObjectsInTestContainer(conn, client);
            DeleteAllObjectsInTestContainer(conn, client);
            DeleteContainer(conn, client);

            Console.ReadLine();
        }

        private static void DeleteAllObjectsInTestContainer(KT_Connection conn, KT_Client client)
        {
            var container = new KT_Container(conn, client, TestContainerName);
            var list = container.GetObjectList();
            Console.WriteLine("Deleting all objects in {0} container", TestContainerName);
            foreach (var item in list)
            {
                container.DeleteObject(item[NameKey]);
            }
            Console.WriteLine("Deleted all objects in {0} container", TestContainerName);
            BreakLine();
        }

        private static void DeleteContainer(KT_Connection conn, KT_Client client)
        {
            try
            {
                var account = new KT_Account(conn, client);
                Console.WriteLine("Deleting container {0}", TestContainerName);
                account.DeleteContainer(TestContainerName);
                Console.WriteLine("Successfully deleted container {0}", TestContainerName);    
            }
            catch(Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
            BreakLine();
        }

        private static void CreateContainer(KT_Connection conn, KT_Client client)
        {
            var account = new KT_Account(conn, client);
            Console.WriteLine("Creating container {0}", TestContainerName);
            account.CreateContainer(TestContainerName);
            Console.WriteLine("Successfully created container {0}", TestContainerName);
            BreakLine();
        }

        private static void ListAllContainers(KT_Connection conn, KT_Client client)
        {
            var account = new KT_Account(conn, client);
            var list = account.GetContainers(true);
            Console.WriteLine("There are {0} containers in the account", list.Count);
            Console.WriteLine("Listing all containers");
            foreach (var item in list)
            {
                Console.WriteLine(item.Name);
            }
            Console.WriteLine("Listing all containers complete");
            BreakLine();
        }

        private static void ListAllObjectsInTestContainer(KT_Connection conn, KT_Client client)
        {
            var container = new KT_Container(conn, client, TestContainerName);
            var list = container.GetObjectList(true);
            Console.WriteLine("There are {0} items in the {1} container", list.Count, TestContainerName);
            Console.WriteLine("Listing all objects in {0} container", TestContainerName);
            foreach(var item in list)
            {
                Console.WriteLine(item[NameKey]);
                Console.WriteLine("=============");
            }
            Console.WriteLine("Listed all objects in {0} container", TestContainerName);
            BreakLine();
        }

        private static void SaveObjectLocally(KT_Connection conn, KT_Client client)
        {
            var container = new KT_Container(conn, client, TestContainerName);
            var obj = new KT_Object(conn, container, client, ObjectName);
            const string file_name = @"C:\test\" + LocalFileName;
            Console.WriteLine("Saving File {0} locally", LocalFileName);
            obj.SaveToFile(file_name, true);
            Console.WriteLine("File {0} saved locally", LocalFileName);
            BreakLine();
        }

        private static void SaveObjectToCloud(KT_Connection conn, KT_Client client)
        {
            var container = new KT_Container(conn, client, TestContainerName);
            var obj = new KT_Object(conn, container, client, ObjectName);
            Console.WriteLine("Saving File {0} to Cloud", LocalFileName);
            obj.WriteFromFile(@"C:\test\" + LocalFileName);
            Console.WriteLine("File {0} saved to Cloud with object name {1}", LocalFileName, ObjectName);
            BreakLine();
        }

        private static void BreakLine()
        {
            Console.WriteLine("========================================================");
            Console.WriteLine("========================================================");
        }      
    }
}